// Taken from <http://ejohn.org/projects/flexible-javascript-events/>
function addEvent( obj, type, fn ) {
  if ( obj.attachEvent ) {
    obj['e'+type+fn] = fn;
    obj[type+fn] = function(){obj['e'+type+fn]( window.event );}
    obj.attachEvent( 'on'+type, obj[type+fn] );
  } else
    obj.addEventListener( type, fn, false );
}
function removeEvent( obj, type, fn ) {
  if ( obj.detachEvent ) {
    obj.detachEvent( 'on'+type, obj[type+fn] );
    obj[type+fn] = null;
  } else
    obj.removeEventListener( type, fn, false );
}

Object.prototype.addEvent = function (type, fn) {
	addEvent(this, type, fn);
}

Object.prototype.removeEvent = function(type, fn) {
	removeEvent(this, type, fn);
}

Object.prototype.fadeout = function () {
	var obj = this;
	var i = 1.0;
	var val = window.setInterval(function () {
		i -= 0.05;
		if (i < 0.0) {
			window.clearInterval(val);
			obj.style.display = 'none';
		}
		else { obj.style.opacity = i.toString(); }
	}, 10);
}

Object.prototype.fadein = function () {
	var obj = this;
	var i = 0.0;
	obj.style.opacity = '0';
	obj.style.display = 'block';
	var val = window.setInterval(function () {
		i += 0.05;
		if (i > 1.0) {
			window.clearInterval(val);
		}
		else { obj.style.opacity = i.toString(); }
	}, 10);
}

function $(str) { return document.getElementById(str); }

// ===========
// = Strings =
// ===========

String.prototype.lastPathComponent = function() {
	var pathComponents = this.split("/");
	for (var i = pathComponents.length; i > 0; i--)
	{
		if (pathComponents[i] != '' && pathComponents[i] != null) 
			return pathComponents[i];
	}
}

String.prototype.stringByDeletingLastPathComponent = function() {
	var idx = this.indexOf(this.lastPathComponent());
	if (idx == -1) return "";
	return this.substring(0, idx);
}

String.prototype.pathExtension = function() {
	var idx = this.lastIndexOf(".");
	if (idx == -1) return "";
	return this.substring(idx, this.length);
}

String.prototype.trim = function () {
	for (var i=0; i < this.length; i++) {
		if (this[i] != ' ') break;
	};
	return this.substr(i);
};

String.prototype.path = function () {
	var tmp = this.replace("://", "");
	var pathComponents = tmp.split("/");
	pathComponents.shift();
	return pathComponents.join("/");
}

String.prototype.contains = function (str) {
	return this.search(str) > -1;
}

// ===========
// = Classes =
// ===========

function Cookie () {
	var self = this;
	this._keys = new Array();
	this._dict = fcsparsecookie();
	this.toDict = function () { return this._dict; }
	this.valueForKey = function (key) { return this._dict[key]; }
	this.setValue_forKey_ = function(value, key) {
		self._dict[key] = value;
		fcscookieadduniquekey(key);
	}

	this.save = function() {
		var x = '';
		for (var i=0; i < self._keys.length; i++) {
			document.cookie = self._keys[i] + '=' + self._dict[self._keys[i]] + ';';
		};
	}
	
	function fcsparsecookie() {
		var x = document.cookie;
		if (x == null || x == '') return {};
		var y = x.split(';');
		var ret = {};
		for (var i=0; i < y.length; i++) {
			var z = y[i].split('=');
			var key = z[0].trim();
			ret[key] = z[1];
			self._keys[self._keys.length] = key;
		};
		return ret;
	}
	
	function fcscookieadduniquekey (key) {
		for (var i=0; i < self._keys.length; i++) {
			if (self._keys[i] == key) return;
		};
		self._keys[self._keys.length] = key;
	}
}

function AsyncRequest () {
	var self = this;
	this.request = null;
	this.action = null;
	this.userdata = null;
	
	this.fetch = function (url) {
		if (window.XMLHttpRequest) self.request = new XMLHttpRequest();
		else if (window.ActiveXObject) 
		{	// I hate WinIE.  This was adapted from <http://developer.apple.com/internet/webcontent/xmlhttpreq.html>
			try { self.request = new ActiveXObject("Msxml2.XMLHTTP"); }
			catch (e) { self.request = new ActiveXObject("Microsoft.XMLHTTP"); }
		}
		self.request.onload = function (e) { self.action(e, self); };
		self.request.open("GET", url);
		self.request.send(null);
	}
}

function Alert(title, msg) {
	var self = this;
	this.title = title;
	this.msg = msg;
	this.elem = null;
	
	function init () {
		var elem = document.createElement("fcsalert");
		elem.style.display = 'none';
		elem.style.background = 'url(/app/color/windowBackgroundColor)';
		elem.style.position = 'absolute';
		elem.style.padding = '1em';
		elem.style.width = '60%';
		elem.style.left = '20%';
		elem.style.top = '20%';
		elem.style.border = '1px solid #000';
		
		var h1 = document.createElement("h1");
		h1.style.fontSize = "1.2em";
		h1.style.margin = "0.2em 0";
		h1.appendChild(document.createTextNode(self.title));
		elem.appendChild(h1);
		
		var p = document.createElement("p");
		p.appendChild(document.createTextNode(self.msg));
		elem.appendChild(p);

		document.body.appendChild(elem);
		self.elem = elem;
	}
	
	// Buttons is an array of dictionaries with keys "title" & "action"
	this.setButtons = function(buttons){
		var p = document.createElement("p");
		this.elem.appendChild(p);
		p.style.textAlign = "right";
		p.style.marginBottom = '0';
		for (var idx = buttons.length - 1; idx >= 0; idx--){
			var b = buttons[idx];
			var i = document.createElement("input");
			i.type = "submit";
			i.value = b['title'];
			i.addEventListener('click', b['action'], false);
			p.appendChild(i);
		};
	}
	
	this.show = function () {
		this.elem.fadein();
	}
	
	this.hide = function() {
		this.elem.fadeout();
	}
	
	this.close = this.hide;
	
	init();
}